#!/usr/bin/env bash
# Centova Cast Upgrade Script
# Copyright 2007, Centova Technologies Inc.
# ============================================================================
#
# OPTIONS
#
# If you have your PHP CLI binary installed in an unusual location, set the
# pathname here.
#
# NOTE: This does not need to be modified if your PHP CLI binary is found in
# your search path.
PHPBINARY=php

# Set this to your new license key, if applicable
NEWLICENSEKEY=

# ============================================================================
# Upgrader code follows.
# You should not need to edit anything below this line.

DEFUSERNAME=centovacast
IGNOREDEPS=0
FORCEWEBROOT=""
PACKAGEUPGRADE=0

while [ "$1" != "${1##[-+]}" ]; do
	case $1 in
		--php=?*)
			PHPBINARY=${1#--php=}
			shift
			;;
		--webroot=?*)
			FORCEWEBROOT=${1#--webroot=}
			shift
			;;
		--packageupgrade)
			PACKAGEUPGRADE=1
			shift
			;;			
        *)
        	echo "Unrecognized option: $1"
        	exit 1
			;;
	esac
done


function verify_exists {
	[ $IGNOREDEPS -gt 0 ] && return
	TESTFILE=`which $1`
	if [ -z "$TESTFILE" ]; then
		echo "Error: Installer requires a working copy of $2 in your path."
		[ ! -z "$2" ] && echo $2
		exit 1
	fi
}


MYUID=`id -u`
[ $MYUID -eq 0 ] && ISROOT=1 || ISROOT=0

# are we FreeBSD? (todo: add support for other BSDs)
KERNOSTYPE=`/sbin/sysctl -n kern.ostype 2>&1`
[ "$KERNOSTYPE" == "FreeBSD" ] && ISBSD=1 || ISBSD=0

if [ $ISBSD -gt 0 ]; then
	# setup BSD utility paths
	STATCMD="stat -f '%Su:%Sg'"
else
	# setup Linux utility paths
	STATCMD="stat -c '%U:%G'"
fi

if [ "$PHPBINARY" == "php" ]; then
	verify_exists php "the PHP CLI binary" "Please specify the path to php using the --php parameter, eg:

$0 $* --php=/path/to/php"


else
	if [ ! -x "$PHPBINARY" ]; then
		echo "$PHPBINARY was not found.  Please specify the path to
php using the --php parameter, eg:

$0 $* --php=/path/to/php
"
		exit 1
	fi
fi

if [ "$PHPBINARY" == "php" ]; then
	[ -f /usr/bin/which ] && WHICHPHP=`which php` || WHICHPHP=""
	[[ "$WHICHPHP" == "/bin/php" || "$WHICHPHP" == "/usr/bin/php" || "$WHICHPHP" == "/usr/local/bin/php" ]] && WHICHPHP=""

	PHPFOUND=0
	PHPMULTI=0
	PHPMULTIPATHS=""
	PHPPATHS="/bin/php /usr/bin/php /usr/local/bin/php $WHICHPHP"

	for f in $PHPPATHS; do
		if [ -f $f ]; then
			if [ $PHPFOUND -gt 0 ]; then
				PHPMULTI=1
			else
				PHPFOUND=1
				PHPBINARY=$f
			fi
			PHPIDENT=`$f -v | grep '^PHP'`
			PHPMULTIPATHS="$PHPMULTIPATHS
$f - $PHPIDENT"
		fi
	done
	
	if [ $PHPFOUND -eq 0 ]; then
		echo "
The PHP CLI binary was not found on your server.  Please specify the path to
php using the --php parameter, eg:

$0 $* --php=$PHPBINARY
"
		exit 1
	fi
	
	if [ $PHPMULTI -gt 0 ]; then
		echo "
More than one PHP commandline interpreter (CLI) was found on this server in the
following locations:
$PHPMULTIPATHS

Please re-run this upgrade script with the --php parameter to specify the path
to the correct PHP CLI which matches your web server's version of PHP.
For example:

$0 $* --php=$PHPBINARY

This will ensure the correct operation of Centova Cast.
"
		exit 1
	fi
	
fi

MYUSER=`whoami`
MYCWD=`pwd`
SELFPATH=`dirname "$0"`
SELFPATH=`echo "${MYCWD}/${SELFPATH}" | sed 's/\/\+/\//g' | sed 's/\/\.//g'`
[ -d /usr/local/cpanel ] && HASCPANEL=1 || HASCPANEL=0
[ -d /usr/local/psa ] && HASPLESK=1 || HASPLESK=0

[ $ISROOT -gt 0 ] && SAMPLEUSER=username || SAMPLEUSER=$MYUSER

CPNAME=""
if [ $HASCPANEL -gt 0 ]; then
	CPNAME="CPanel"
	SAMPLEROOT="/home/$SAMPLEUSER/public_html"
elif [ $HASPLESK -gt 0 ]; then
	[ -d /home/httpd/vhosts ] && PLESKROOT=/home/httpd
	[ -d /var/www/vhosts ] && PLESKROOT=/var/www
	CPNAME="Plesk"
	SAMPLEROOT="$PLESKROOT/domain.com/httpdocs"
else
	CPNAME="Apache"
	SAMPLEROOT="/var/www/html"
fi

[ ! -f "$SELFPATH/system/class_SystemInterface.php" ] && echo "Cannot find unpacked system/ directory." && exit 1
[ ! -f "$SELFPATH/www/class_AccountEditor.php" ] && echo "Cannot find unpacked www/ directory." && exit 1


echo "==============================================================================="
echo "Centova Cast Upgrade"
echo "==============================================================================="
echo ""
echo "This script will upgrade your existing Centova Cast installation."
echo ""
echo "**** WARNING ****"
echo "BEFORE PROCEEDING, BE SURE TO BACK UP YOUR CENTOVA CAST DATABASE AND ALL OF"
echo "YOUR CENTOVA CAST FILES, including those in your main Centova Cast directory"
echo "(usually /usr/local/centovacast/ or /home/centovacast/) as well as those"
echo "under your web root (eg: $SAMPLEROOT/cast/ or wherever you installed it)."
echo "**** WARNING ****"
echo ""
echo "If you have not already created your backups, press Ctrl+C to cancel the"
echo "upgrade and do so now.  Otherwise, press ENTER to continue."
echo ""
echo "==============================================================================="
echo ""
echo ""
read

WEBROOT=""
while [ -z "$WEBROOT" ]; do
	if [ ! -z "$FORCEWEBROOT" ]; then
		# use web root specified as commandline argument
		echo "Using web interface path: $FORCEWEBROOT"
		echo ""
		
		WEBROOT="$FORCEWEBROOT"
		FORCEWEBROOT=""
	else
		# prompt for web root
		echo "Please enter the path to your Centova Cast control panel files.  This"
		echo "directory will be a subdirectory in your web root, and will contain"
		echo "files named 'tunein.php' and 'api.php' among others."
		echo ""
		echo "For example, under ${CPNAME}, your web root will usually be:"
		echo "     $SAMPLEROOT"
		echo ""
		echo "So the path to your Centova Cast control panel files might be something like:"
		echo "     $SAMPLEROOT/cast"
		echo "Or:"
		echo "     $SAMPLEROOT/centovacast"
		echo ""

		echo -n "Enter the path to the Centova Cast control panel files: "
		read WEBROOT
	fi
	echo ""
	echo ""
	echo "==============================================================================="
	if [ -z "$WEBROOT" ]; then
		echo "You must specify the path to your Centova Cast control panel files."
		echo ""
	elif [ ! -f "$WEBROOT/class_AccountEditor.php" ]; then
		
		# if it didn't exist, try a few variations
		ORIGROOT="$WEBROOT"
		if   [ -f "$WEBROOT/public_html/class_AccountEditor.php" ]; then
		   WEBROOT=$WEBROOT/public_html/
		elif [ -f "$WEBROOT/public_html/cast/class_AccountEditor.php" ]; then
		   WEBROOT=$WEBROOT/public_html/cast/
		elif [ -f "$WEBROOT/public_html/centovacast/class_AccountEditor.php" ]; then
		   WEBROOT=$WEBROOT/public_html/centovacast/
		elif [ -f "$WEBROOT/www/class_AccountEditor.php" ]; then
		   WEBROOT=$WEBROOT/www/
		elif [ -f "$WEBROOT/www/cast/class_AccountEditor.php" ]; then
		   WEBROOT=$WEBROOT/www/cast/
		elif [ -f "$WEBROOT/www/centovacast/class_AccountEditor.php" ]; then
		   WEBROOT=$WEBROOT/www/centovacast/
		elif [ -f "$WEBROOT/httpdocs/class_AccountEditor.php" ]; then
		   WEBROOT=$WEBROOT/httpdocs/
		elif [ -f "$WEBROOT/httpdocs/cast/class_AccountEditor.php" ]; then
		   WEBROOT=$WEBROOT/httpdocs/cast/
		elif [ -f "$WEBROOT/httpdocs/centovacast/class_AccountEditor.php" ]; then
		   WEBROOT=$WEBROOT/httpdocs/centovacast/
		else
			echo "The specified path does not appear to contain the Centova Cast control panel"
			echo "files.  Please try again."
			WEBROOT=""
		fi
		if [ ! -z "$WEBROOT" ]; then
			echo ""
			echo "Centova Cast was not found in: $ORIGROOT"
			echo ""
			echo "But a Centova Cast installation was found in the following directory:"
			echo "  $WEBROOT"
			echo ""
			echo -n "Press ENTER to proceed with upgrading this installation, or Ctrl+C to abort."
			read
		fi
	else
		echo "Found Centova Cast in $WEBROOT/"
	fi
	echo "==============================================================================="
	echo ""
	echo ""
done

# verify that the web root contains a valid config.php that points to the system
# path for the Centova Cast installation

DEFSYSPATH="/home/centovacast/system/"
[ -d /usr/local/centovacast/system ] && DEFSYSPATH="/usr/local/centovacast/system/"

echo "<?php
@error_reporting(E_ALL ^ E_NOTICE);
require('$WEBROOT/config.php');
define('SYSTEM_PATH','$DEFSYSPATH');
require(SYSTEM_PATH.'config.php');
require(dirname(__FILE__).'/system/upgradeutil.php');
?>" > upgrade.php

rm -f $SELFPATH/system/config.php
touch $SELFPATH/system/config.php
rm -f $SELFPATH/www/config.php
touch $SELFPATH/www/config.php

rmdir $SELFPATH/ioncube >/dev/null 2>&1
[ ! -d $SELFPATH/ioncube ] && ln -s $WEBROOT/ioncube $SELFPATH/ioncube

CHK=`$PHPBINARY -q upgrade.php --test 2>&1`
if [ "x$CHK" != "xOK" ]; then
	echo "The specified path appears to contain Centova Cast files, but this upgrade"
	echo "script encountered an error when attempting to use them, and thus cannot"
	echo "proceed with the upgrade."
	echo ""
	echo "The error message was:"
	echo "$CHK"
	echo ""
	rm -f upgrade.php
	exit 1
fi

CHK=`$PHPBINARY -q upgrade.php --sanitycheck 2>&1`
if [ "x$CHK" != "xOK" ]; then
	echo "The specified path appears to contain Centova Cast files, but one or more"
	echo "problem(s) were found:"
	echo "    $CHK"
	echo "Upgrade cannot proceed."
	rm -f upgrade.php
	exit 1
fi


SYSPATH=`$PHPBINARY -q upgrade.php --syspath 2>&1`

OLDMAJORVER=`${SYSPATH}runascc/runascc exec ccmanage version all | grep '^OK' | sed 's/.*v//g' | sed 's/\..*//g'`
OLDMINORVER=`${SYSPATH}runascc/runascc exec ccmanage version all | grep '^OK' | sed 's/.*v[0-9]\.//g' | sed 's/\..*//g'`

if [ "$OLDMAJORVER" -gt 2 ]; then
	HASSANITY=1
elif [[ "$OLDMAJORVER" -gt 1 && "$OLDMINORVER" -gt 0 ]]; then
	HASSANITY=1
else
	HASSANITY=0
fi

if [ $HASSANITY -gt 0 ]; then
	CCMRESULT=`${SYSPATH}runascc/runascc exec ccmanage sanitycheck nopassword`
	CCMOK=`echo "$CCMRESULT" | grep '^OK'`
		if [ -z "$CCMOK" ]; then
		echo "The specified path appears to contain Centova Cast files, but one or more"
		echo "problem(s) were found:"
		echo "    $CCMRESULT"
		echo "Upgrade cannot proceed."
		rm -f upgrade.php
		exit 1
	fi
fi

OLDMAJORVER=`${SYSPATH}runascc/runascc exec ccmanage version all | grep '^OK' | sed 's/.*v//g' | sed 's/\..*//g'`

echo "Updating database ..."

if [ ! -z "$NEWLICENSEKEY" ]; then
	NEWLICENSEKEY="--newlicense=$NEWLICENSEKEY"
fi

RES=`$PHPBINARY -q upgrade.php --upgrade $NEWLICENSEKEY 2>&1`
if [ "x$RES" != "xOK" ]; then
	echo "One or more error(s) occurred while attempting to update your database:"
	echo ""
	echo "$RES"
	echo ""
	echo "Upgrade failed.  You will need to restore from backup and then try the"
	echo "upgrade procedure again from the start."
	rm -f upgrade.php
	exit 1
fi


# get current owner of Centova Cast web interface files (needed to maintain ownerships for suPHP)
WEBOWNER=`$STATCMD $WEBROOT/index.php 2>/dev/null`

echo "Installing new files ..."

rm -f $SELFPATH/system/config.php
rm -f $SELFPATH/www/config.php
mv -f $SELFPATH/system/runascc $SELFPATH/sys_runascc

cp -rf $SELFPATH/system/* "$SYSPATH"
mv -f $SELFPATH/sys_runascc $SELFPATH/system/runascc
cp -rf $SELFPATH/ioncube/* "$SYSPATH/../ioncube/"

rm -f $WEBROOT/system/*.js
rm -f $WEBROOT/admin/class_*.php $WEBROOT/admin/DEPR*.php $WEBROOT/admin/index.html $WEBROOT/admin/.htaccess
rm -f $WEBROOT/modules/class_Module_*.php

# before copying web interface into place, chown it to the owner of the current web interface
# files to appease suPHP
chown -R $WEBOWNER $SELFPATH/www/* >/dev/null 2>&1
chgrp -R $WEBOWNER $SELFPATH/www/* >/dev/null 2>&1

cp -rf $SELFPATH/www/* "$WEBROOT/"

mkdir -p "$SYSPATH/../scripts/"
cp -rf $SELFPATH/scripts/* "$SYSPATH/../scripts/"

chmod og+x ${SYSPATH}../vhosts/*/var/log


if [ ! -z "$NEWLICENSEKEY" ]; then

	while [ true ]; do
		echo ""
		echo "Updating license ... "
		echo ""	
		CCLRESULT=`${SYSPATH}runascc/runascc exec ccmanage reissuelicense @current`
		CCLOK=`echo "$CCLRESULT" | grep '^OK'`
		if [ -z "$CCMOK" ]; then
			echo "Your Centova Cast license key could not be updated:"
			echo ""
			echo "$CCLRESULT" | grep '^ERR'
			echo ""
			echo "In most cases a licensing error at this point means that you forgot to"
			echo "click the 'Re-issue license' link in your client area at centova.com"
			echo "before upgrading."
			echo ""
			echo "Your upgrade is NOT complete, however it cannot proceed until this problem"
			echo "is resolved.  If you cannot resolve this problem and need to abort, you"
			echo "will need to restore from backup and try the upgrade procedure again from"
			echo "the start."
			echo ""
			echo "Press Ctrl+C to abort, or press ENTER to try again."
			echo ""
			read -s
		else
			break;
		fi
	done

fi

while [ true ]; do
	CCMRESULT=`${SYSPATH}runascc/runascc exec ccmanage sanitycheck nopassword`
	CCMOK=`echo "$CCMRESULT" | grep '^OK'`
	if [ -z "$CCMOK" ]; then
		echo "A problem with your Centova Cast installation has been detected:"
		echo ""
		echo $CCMRESULT
		echo ""
		echo "Your upgrade is NOT complete, however it cannot proceed until this problem"
		echo "is resolved.  If you cannot resolve this problem and need to abort, you"
		echo "will need to restore from backup and try the upgrade procedure again from"
		echo "the start."
		echo ""
		echo "Press Ctrl+C to abort, or press ENTER to try again."
		echo ""
		read -s
	else
		break;
	fi
done


if [ "$OLDMAJORVER" != "2" ]; then
	echo ""
	echo "Centova Cast 2.0 includes a new media library feature which allows clients"
	echo "to easily browse and search their media.  Before use, each client's media"
	echo "files must be indexed."
	echo ""
	
	ADMINPASS=""
	
	while [ -z "$ADMINPASS" ]; do
		echo "Enter the Centova Cast administrator password to begin the indexing"
		echo "process."
		echo ""
		echo -n "Administrator password (will not be echoed): "
		read -s ADMINPASS
		echo ""
		echo ""
		echo ""
		PASSRESULT=`echo "$ADMINPASS" | ${SYSPATH}runascc/runascc exec ccmanage info null`
		PASSOK=`echo "$PASSRESULT" | grep 'OK'`
		if [ -z "$PASSOK" ]; then
			echo "Sorry, that password was incorrect.  Please try again."
			echo ""
			ADMINPASS=""
		fi
	done
	
	echo "Indexing media libraries (this may take a while) ..."
	echo ""
	
	ACCTLIST=`echo "$ADMINPASS" | ${SYSPATH}runascc/runascc exec ccmanage info all | grep '^DAT' | sed 's/^DAT username=//g' | sed 's/|.*//g'`
	for f in $ACCTLIST; do
		echo "Processing account $f ..."
	
		# setup on-demand directories while we're here
		ACCTDIR="${SYSPATH}../vhosts/${f}/"
		if [ ! -d ${ACCTDIR}var/spool/ondemand ]; then
			SPOOLOWNER=`$STATCMD ${ACCTDIR}var/spool 2>/dev/null`
			mkdir ${ACCTDIR}var/spool/ondemand >/dev/null 2>&1
			chown $SPOOLOWNER ${ACCTDIR}var/spool/ondemand >/dev/null 2>&1
			ln -s ${ACCTDIR}var/spool/ondemand ${ACCTDIR}web/ondemand >/dev/null 2>&1
		fi
	
	
		# then reindex	
		echo ""
		echo "admin|$ADMINPASS" | ${SYSPATH}runascc/runascc exec ccmanage reindex $f --intoplaylistname=standard --ignoremissingplaylist=1
		echo ""
	done
fi

rm -f upgrade.php

echo "Upgrade completed successfully."
echo ""
echo "Note that if you have installed the Centova Cast billing or FTP integration"
echo "modules you will need to install the latest module updates from the billing/"
echo "and ftp/ directories now."
echo ""
echo "Thank you for using Centova Cast."
exit 0


